<?php

/**
 * Plugin Name:       Fast Backend
 * Plugin URI:        https://www.webdesign-muenchen.de/wordpress-plugin-fast-backend/
 * Description:       Improve Speed of WordPress Admin Backend
 * Version:           1.5.1
 * Requires at least: 5.0
 * Requires PHP:      7.2
 * Author:            WDM
 * Author URI:        https://www.webdesign-muenchen.de
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 */

// Reduce WP Heartbeat 
function fast_backend_heartbeat_settings( $settings ) {
    $settings['interval'] = 45;
    return $settings;
}
add_filter('heartbeat_settings','fast_backend_heartbeat_settings');

function fast_backend_optimize_stylesheets ( $html ) {
    preg_match_all( '#\n?<link rel=\'stylesheet\' id=\'(.+)\' +href=\'(.+)\'.+>#', $html, $matches );
    if ( $matches && count( $matches[ 0 ] ) ) {
        list( $links, $ids, $hrefs ) = $matches;
        /**
         * Filters the IDs of stylesheets to remove entirely from loading
         *
         * @since 1.5
         *
         * @param array $stylesheet_ids  The IDs of stylesheets to be removed 
         */
        $removeIds = apply_filters( 'fast_backend_stylesheets_remove', [] );
        /**
         * Filters the IDs of stylesheets to be loaded in the <head>
         *
         * @since 1.5
         *
         * @param array $stylesheet_ids  The IDs of stylesheets to load in the <head>
         */
        $inHeadIds = apply_filters( 'fast_backend_stylesheets_in_head', ['acf-global-css','acf-input-css','acf-pro-input-css'] );
        /**
         * Filters the IDs of stylesheets to be loaded at the bottom of the <body>
         *
         * @since 1.5
         *
         * @param array $stylesheet_ids  The IDs of stylesheets to load at the bottom of the <body>
         */
        $inFooterIds = apply_filters( 'fast_backend_stylesheets_in_footer', [] );
        /**
         * Filters the IDs of stylesheets to be lazy loaded after the page has loaded
         *
         * @since 1.5
         *
         * @param array $stylesheet_ids  The IDs of stylesheets lazy load
         */
        $lazyLoadIds = apply_filters( 'fast_backend_stylesheets_lazy_load', [] );
        $defers = [];
        $lazyLoads = [];

        foreach ( $links as $index => $link ) {
            $id = $ids[ $index ];
            $isPluginStylesheet = preg_match( '#'.WP_CONTENT_URL.'/plugins/#', $hrefs[ $index ] );
            if ( in_array( $id, $removeIds ) ) {
                $html = preg_replace( '#'.preg_quote( $link ).'#', '', $html );
            } else if ( in_array( $id, $lazyLoadIds ) ) {
                $html = preg_replace( '#'.preg_quote( $link ).'#', '', $html );
                $lazyLoads[] = str_replace( "\n", '', $link );
            } else if ( !in_array( $id, $inHeadIds ) && ( $isPluginStylesheet || in_array( $id, $inFooterIds ) ) ) {
                $html = preg_replace( '#'.preg_quote( $link ).'#', '', $html );
                $defers[] = str_replace( [ ' media=\'all\'', "\n" ], [ ' media=\'lazy\' onload=\'this.media="all"\'', '' ], $link );
            }
        }

        $htmlStyles = '';
        if ( count( $defers ) ) {
            $htmlStyles .= implode( "\n", $defers )."\n";
        }
        if ( count( $lazyLoads ) ) {
            $htmlStyles .= '<script>';
            $htmlStyles .=   'setTimeout(function () {';
            $htmlStyles .=     'var el = document.createElement("div");';
            $htmlStyles .=     'el.innerHTML = `'.implode( '', $lazyLoads ).'`;';
            $htmlStyles .=     'document.body.appendChild(el);';
            $htmlStyles .=   '}, 2000);';
            $htmlStyles .= '</script>';
        }

        $html = str_replace( '</body>', $htmlStyles.'</body>', $html );
    }
    return $html;
}

function fast_backend_optimize_imgs ( $html ) {
    preg_match_all( '#<img .+>#', $html, $matches );
    foreach ( $matches[ 0 ] as $img ) {
        // match attribute escaping in <img>
        $quote = preg_match( '#=\\\\"#', $img ) ? '\\"' : '"';
        $adjusted = $img;
        if ( strpos( $img, ' decoding=' ) === false ) {
            $adjusted = str_replace( '<img ', '<img decoding='.$quote.'async'.$quote.' ', $adjusted );
        }
        if ( strpos( $img, ' loading=' ) === false ) {
            $adjusted = str_replace( '<img ', '<img loading='.$quote.'lazy'.$quote.' ', $adjusted );
        }
        if ( $adjusted !== $img ) {
            $html = str_replace( $img, $adjusted, $html );
        }
    }
    return $html;
}

function fast_backend_backendbuffer($html) {
    $html = fast_backend_optimize_stylesheets( $html );

    // Add native lazy loading for images in backend only for Post Type Overview page, but not for new pages,
    // or only for Post edit screen, but not when Gutenberg (Block Editor) is enabled
    $isPostTypeOverviewPage = isset($_GET['post_type']) && array_pop( explode( '/', $_SERVER[ 'PHP_SELF' ] ) ) !== 'post-new.php';
    $isNonGutenbergPostEditPage = isset($_GET['action']) && $_GET['action'] == 'edit' && isset($_GET['post']) && !use_block_editor_for_post($_GET['post']);
    if ( $isPostTypeOverviewPage || $isNonGutenbergPostEditPage ) {
        $html = fast_backend_optimize_imgs( $html );
    }
    return $html;
}

// Perform optimizations in Backend
function fast_backend_backend_buffer_start() { 
    // But not, if an AJAX call is performed
    if (!defined('DOING_AJAX') || !DOING_AJAX) {
        ob_start('fast_backend_backendbuffer');
    }
}
add_action('admin_init','fast_backend_backend_buffer_start',10,0);

// Increase Autosave Interval to 2 minutes
if (!defined('AUTOSAVE_INTERVAL')) { define( 'AUTOSAVE_INTERVAL', 120 ); }
